@@ -4,6 +4,8 @@
 #include <sys/types.h>
 #include <glob.h>
 
+#include "config.h"
+
 #define LOG_FLAG_COMPRESS	(1 << 0)
 #define LOG_FLAG_CREATE		(1 << 1)
 #define LOG_FLAG_IFEMPTY	(1 << 2)
@@ -12,18 +14,11 @@
 #define LOG_FLAG_MISSINGOK	(1 << 5)
 #define LOG_FLAG_MAILFIRST	(1 << 6)
 #define LOG_FLAG_SHAREDSCRIPTS	(1 << 7)
-
-#define DEFAULT_MAIL_COMMAND "/bin/mail -s"
-#define COMPRESS_COMMAND "/bin/gzip"
-#define COMPRESS_OPTIONS "-9"
-#define COMPRESS_EXT ".gz"
-#define UNCOMPRESS_COMMAND "/bin/gunzip"
+#define LOG_FLAG_COPY		(1 << 8)
 
 #define NO_FORCE_ROTATE 0
 #define FORCE_ROTATE    1
 
-#define STATEFILE "/var/lib/logrotate.status"
-
 typedef struct {
     char * pattern;
     char ** files;
@@ -32,6 +27,7 @@ typedef struct {
     enum { ROT_DAYS, ROT_WEEKLY, ROT_MONTHLY, ROT_SIZE, ROT_FORCE } criterium;
     unsigned int threshhold;
     int rotateCount;
+    int logStart;
     char * pre, * post;
     char * logAddress;
     char * extension;
